@@ -3,7 +3,6 @@
 import re
 import string
 
-from faker.generator import random
 from faker.utils.distribution import choice_distribution
 
 
@@ -22,8 +21,7 @@ class BaseProvider(object):
     def __init__(self, generator):
         self.generator = generator
 
-    @classmethod
-    def random_int(cls, min=0, max=9999):
+    def random_int(self, min=0, max=9999):
         """
         Returns a random integer between two values.
 
@@ -31,66 +29,65 @@ class BaseProvider(object):
         :param max: upper bound value (inclusive; default=9999)
         :returns: random integer between min and max
         """
-        return random.randint(min, max)
+        return self.generator.random.randint(min, max)
 
-    @classmethod
-    def random_digit(cls):
+    def random_digit(self):
         """
         Returns a random digit/number
         between 0 and 9.
         """
-        return random.randint(0, 9)
+        return self.generator.random.randint(0, 9)
 
-    @classmethod
-    def random_digit_not_null(cls):
+    def random_digit_not_null(self):
         """
         Returns a random non-zero digit/number
         between 1 and 9.
         """
-        return random.randint(1, 9)
+        return self.generator.random.randint(1, 9)
 
-    @classmethod
-    def random_digit_or_empty(cls):
+    def random_digit_or_empty(self):
         """
         Returns a random digit/number
         between 0 and 9 or an empty string.
         """
-        if random.randint(0, 1):
-            return random.randint(0, 9)
+        if self.generator.random.randint(0, 1):
+            return self.generator.random.randint(0, 9)
         else:
             return ''
 
-    @classmethod
-    def random_digit_not_null_or_empty(cls):
+    def random_digit_not_null_or_empty(self):
         """
         Returns a random non-zero digit/number
         between 1 and 9 or and empty string.
         """
-        if random.randint(0, 1):
-            return random.randint(1, 9)
+        if self.generator.random.randint(0, 1):
+            return self.generator.random.randint(1, 9)
         else:
             return ''
 
-    @classmethod
-    def random_number(cls, digits=None):
+    def random_number(self, digits=None, fix_len=False):
         """
-        Returns a random number with 1 digit (default, when digits==None)
-        or a random number with 0 to given number of digits.
+        Returns a random number with 1 digit (default, when digits==None),
+        a random number with 0 to given number of digits, or a random number
+        with given number to given number of digits (when ``fix_len==True``).
 
         :param digits: maximum number of digits
-        :returns: random number with 0 to given number of digits
+        :param fix_len:  should the number have fixed length?
+        :returns: random number with 0 to given number of digits or
+            fixed length number
         """
         if digits is None:
-            digits = BaseProvider.random_digit()
-        return random.randint(0, pow(10, digits) - 1)
+            digits = self.random_digit()
+        if fix_len:
+            return self.generator.random.randint(pow(10, digits - 1), pow(10, digits) - 1)
+        else:
+            return self.generator.random.randint(0, pow(10, digits) - 1)
 
-    @classmethod
-    def random_letter(cls):
+    def random_letter(self):
         """Returns a random letter (between a-z and A-Z)."""
-        return random.choice(getattr(string, 'letters', string.ascii_letters))
+        return self.generator.random.choice(getattr(string, 'letters', string.ascii_letters))
 
-    @classmethod
-    def random_element(cls, elements=('a', 'b', 'c')):
+    def random_element(self, elements=('a', 'b', 'c')):
         """
         Returns a random element from a passed object.
 
@@ -110,34 +107,31 @@ class BaseProvider(object):
         if isinstance(elements, dict):
             choices = elements.keys()
             probabilities = elements.values()
-            return choice_distribution(list(choices), list(probabilities))
+            return choice_distribution(list(choices), list(probabilities), self.generator.random)
         else:
-            return random.choice(list(elements))
+            return self.generator.random.choice(list(elements))
 
-    @classmethod
-    def random_sample(cls, elements=('a', 'b', 'c'), length=None):
+    def random_sample(self, elements=('a', 'b', 'c'), length=None):
         if length is None:
-            length = random.randint(1, len(elements))
+            length = self.generator.random.randint(1, len(elements))
 
-        return [cls.random_element(elements) for _ in range(length)]
+        return [self.random_element(elements) for _ in range(length)]
 
-    @classmethod
-    def random_sample_unique(cls, elements=('a', 'b', 'c'), length=None):
+    def random_sample_unique(self, elements=('a', 'b', 'c'), length=None):
         """
         Returns a `set` of random unique elements for the specified length.
         """
         if length is None:
-            length = random.randint(1, len(elements))
+            length = self.generator.random.randint(1, len(elements))
 
         if length > len(elements):
             raise ValueError("Sample length cannot be longer than the number of elements to pick from.")
         sample = set()
         while len(sample) < length:
-            sample.add(cls.random_element(elements))
+            sample.add(self.random_element(elements))
         return sample
 
-    @classmethod
-    def randomize_nb_elements(cls, number=10, le=False, ge=False):
+    def randomize_nb_elements(self, number=10, le=False, ge=False):
         """
         Returns a random value near number.
 
@@ -150,10 +144,9 @@ class BaseProvider(object):
             return number
         _min = 100 if ge else 60
         _max = 100 if le else 140
-        return int(number * random.randint(_min, _max) / 100) + 1
+        return int(number * self.generator.random.randint(_min, _max) / 100)
 
-    @classmethod
-    def numerify(cls, text='###'):
+    def numerify(self, text='###'):
         """
         Replaces all placeholders in given text with randomized values,
         replacing: all hash sign ('#') occurrences with a random digit
@@ -167,35 +160,33 @@ class BaseProvider(object):
         :returns: string with all numerical placeholders filled in
         """
         text = _re_hash.sub(
-            lambda x: str(BaseProvider.random_digit()),
+            lambda x: str(self.random_digit()),
             text)
         text = _re_perc.sub(
-            lambda x: str(BaseProvider.random_digit_not_null()),
+            lambda x: str(self.random_digit_not_null()),
             text)
         text = _re_excl.sub(
-            lambda x: str(BaseProvider.random_digit_or_empty()),
+            lambda x: str(self.random_digit_or_empty()),
             text)
         text = _re_at.sub(
-            lambda x: str(BaseProvider.random_digit_not_null_or_empty()),
+            lambda x: str(self.random_digit_not_null_or_empty()),
             text)
         return text
 
-    @classmethod
-    def lexify(cls, text='????'):
+    def lexify(self, text='????'):
         """
         Replaces all question mark ('?') occurrences with a random letter.
 
         :param text: string to be parsed
         :returns: string with all letter placeholders filled in
         """
-        return _re_qm.sub(lambda x: BaseProvider.random_letter(), text)
+        return _re_qm.sub(lambda x: self.random_letter(), text)
 
-    @classmethod
-    def bothify(cls, text='## ??'):
+    def bothify(self, text='## ??'):
         """
         Replaces all placeholders with random numbers and letters.
 
         :param text: string to be parsed
         :returns: string with all numerical and letter placeholders filled in
         """
-        return BaseProvider.lexify(BaseProvider.numerify(text))
+        return self.lexify(self.numerify(text))
